#!/usr/bin/env node

/** 
 * 
 * 参考： https://segmentfault.com/a/1190000015467084
 * 优化：通过 X-Forwarded-For 添加了动态随机伪IP，绕过 tinypng 的上传数量限制
 * 
 *  */

const fs = require('fs');
const path = require('path');
const https = require('https');
const {
	URL
} = require('url');
const {
	timeLog
} = require('console');


const cwd = process.cwd();

const root = cwd;
// const root = 'X:/p37/p37/assets/Img/菜品'


console.log('root 目录 ', root);
exts = ['.jpg', '.png'];
const MAX = 5200000; // 5MB == 5242848.754299136

/**
 * 一次最多20个图片
 */
const MAX_COUNT = 20;

let allFiles = [];
let errorFiles = [];

let onceCount = 0;
let onceFinishCount = 0;



const options = {
	method: 'POST',
	hostname: 'tinypng.com',
	path: '/backend/opt/shrink',
	headers: {
		rejectUnauthorized: false,
		'Postman-Token': Date.now(),
		'Cache-Control': 'no-cache',
		'Content-Type': 'application/x-www-form-urlencoded',
		'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36'
	}
};

fileList(root);
TimeLogic();

// 生成随机IP， 赋值给 X-Forwarded-For
function getRandomIP() {
	return Array.from(Array(4)).map(() => parseInt(Math.random() * 255)).join('.')
}

// 获取文件列表
function fileList(folder) {
	DDirfileList(folder);
}

// 获取文件列表
function DDirfileList(folder) {
	let files = fs.readdirSync(folder);
	for (const file of files) {
		const element = path.join(folder, file);
		fileFilter(element);
	}
}

// 过滤文件格式，返回所有jpg,png图片
function fileFilter(file) {
	let stats = fs.statSync(file);
	if (stats.isDirectory()) {
		DDirfileList(file);
	};
	if (
		// 必须是文件，小于5MB，后缀 jpg||png
		stats.size <= MAX &&
		stats.isFile() &&
		exts.includes(path.extname(file))
	) {
		allFiles.push(file);
	}
}


function TimeLogic() {
	onceCount = Math.min(MAX_COUNT, allFiles.length);
	for (let index = 0; index < onceCount; index++) {
		let file = allFiles.shift();
		try {
			let ip = getRandomIP();
			options.headers['X-Forwarded-For'] = ip;
			console.log(`files `, file);
			fileUpload(file);
		} catch (error) {
			errorFiles.push(file);
			finishOnce();
		}
	}

}

/**是否完成一次 */
function finishOnce() {
	onceFinishCount++;
	if (onceFinishCount >= onceCount) {
		onceFinishCount = 0;
		onceCount = 0;
		if (allFiles.length > 0) {
			TimeLogic();
		} else {
			console.log('压缩完毕');
			console.log('压缩错误列表 errorFiles', errorFiles);
			process.exit();
		}
	}
}
// 异步API,压缩图片
function fileUpload(img) {
	var req = https.request(options, function (res) {
		res.on('data', buf => {
			let obj = JSON.parse(buf.toString());
			if (obj.error) {
				errorFiles.push(img);
				console.log(`[${img}]：压缩失败！报错：${obj.message}`);
				finishOnce();
			} else {
				fileUpdate(img, obj);
			}
		});
	});

	req.write(fs.readFileSync(img), 'binary');
	req.on('error', e => {
		errorFiles.push(img);
		finishOnce();
	});
	req.end();
}
// 该方法被循环调用,请求图片数据
function fileUpdate(imgpath, obj) {
	let options = new URL(obj.output.url);
	let req = https.request(options, res => {
		let body = '';
		res.setEncoding('binary');
		res.on('data', function (data) {
			body += data;
		});

		res.on('end', function () {
			fs.writeFile(imgpath, body, 'binary', err => {
				if (err) {
					errorFiles.push(imgpath);
					console.log(`[${imgpath}]：压缩失败！报错：${obj.message}`);
					finishOnce();
					return;
				} else {
					console.log(
						`[${imgpath}] \n 压缩成功，原始大小-${obj.input.size}，压缩大小-${obj.output.size
						}，优化比例-${obj.output.ratio}`
					);
					finishOnce();
				}

			});
		});
	});
	req.on('error', e => {
		errorFiles.push(imgpath);
		finishOnce();
	});
	req.end();
}